package sun.nio.fs;

import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.misc.Unsafe;

class WindowsNativeDispatcher
{
    private static final Unsafe unsafe;
    
    private WindowsNativeDispatcher() {
    }
    
    static native long CreateEvent(boolean p0, boolean p1) throws WindowsException;
    
    static long CreateFile(String s, int n, int n2, long n3, int n4, int n5) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return CreateFile0(nativeBuffer.address(), n, n2, n3, n4, n5);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    static long CreateFile(String s, int n, int n2, int n3, int n4) throws WindowsException {
        return CreateFile(s, n, n2, 0L, n3, n4);
    }
    
    private static native long CreateFile0(long p0, int p1, int p2, long p3, int p4, int p5) throws WindowsException;
    
    static native void CloseHandle(long p0);
    
    static void DeleteFile(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            DeleteFile0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void DeleteFile0(long p0) throws WindowsException;
    
    static void CreateDirectory(String s, long n) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            CreateDirectory0(nativeBuffer.address(), n);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void CreateDirectory0(long p0, long p1) throws WindowsException;
    
    static void RemoveDirectory(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            RemoveDirectory0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void RemoveDirectory0(long p0) throws WindowsException;
    
    static native void DeviceIoControlSetSparse(long p0) throws WindowsException;
    
    static native void DeviceIoControlGetReparsePoint(long p0, long p1, int p2) throws WindowsException;
    
    static WindowsNativeDispatcher.FirstFile FindFirstFile(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            WindowsNativeDispatcher.FirstFile firstFile = new WindowsNativeDispatcher.FirstFile();
            FindFirstFile0(nativeBuffer.address(), firstFile);
            return firstFile;
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void FindFirstFile0(long p0, WindowsNativeDispatcher.FirstFile p1) throws WindowsException;
    
    static long FindFirstFile(String s, long n) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return FindFirstFile1(nativeBuffer.address(), n);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native long FindFirstFile1(long p0, long p1) throws WindowsException;
    
    static native String FindNextFile(long p0, long p1) throws WindowsException;
    
    static WindowsNativeDispatcher.FirstStream FindFirstStream(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            WindowsNativeDispatcher.FirstStream firstStream = new WindowsNativeDispatcher.FirstStream();
            FindFirstStream0(nativeBuffer.address(), firstStream);
            if (firstStream.handle() == -1L) {
                return null;
            }
            return firstStream;
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void FindFirstStream0(long p0, WindowsNativeDispatcher.FirstStream p1) throws WindowsException;
    
    static native String FindNextStream(long p0) throws WindowsException;
    
    static native void FindClose(long p0) throws WindowsException;
    
    static native void GetFileInformationByHandle(long p0, long p1) throws WindowsException;
    
    static void CopyFileEx(String s, String s2, int n, long n2) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        NativeBuffer nativeBuffer2 = asNativeBuffer(s2);
        try {
            CopyFileEx0(nativeBuffer.address(), nativeBuffer2.address(), n, n2);
        }
        finally {
            nativeBuffer2.release();
            nativeBuffer.release();
        }
    }
    
    private static native void CopyFileEx0(long p0, long p1, int p2, long p3) throws WindowsException;
    
    static void MoveFileEx(String s, String s2, int n) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        NativeBuffer nativeBuffer2 = asNativeBuffer(s2);
        try {
            MoveFileEx0(nativeBuffer.address(), nativeBuffer2.address(), n);
        }
        finally {
            nativeBuffer2.release();
            nativeBuffer.release();
        }
    }
    
    private static native void MoveFileEx0(long p0, long p1, int p2) throws WindowsException;
    
    static int GetFileAttributes(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return GetFileAttributes0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native int GetFileAttributes0(long p0) throws WindowsException;
    
    static void SetFileAttributes(String s, int n) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            SetFileAttributes0(nativeBuffer.address(), n);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void SetFileAttributes0(long p0, int p1) throws WindowsException;
    
    static void GetFileAttributesEx(String s, long n) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            GetFileAttributesEx0(nativeBuffer.address(), n);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void GetFileAttributesEx0(long p0, long p1) throws WindowsException;
    
    static native void SetFileTime(long p0, long p1, long p2, long p3) throws WindowsException;
    
    static native void SetEndOfFile(long p0) throws WindowsException;
    
    static native int GetLogicalDrives() throws WindowsException;
    
    static WindowsNativeDispatcher.VolumeInformation GetVolumeInformation(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            WindowsNativeDispatcher.VolumeInformation volumeInformation = new WindowsNativeDispatcher.VolumeInformation();
            GetVolumeInformation0(nativeBuffer.address(), volumeInformation);
            return volumeInformation;
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void GetVolumeInformation0(long p0, WindowsNativeDispatcher.VolumeInformation p1) throws WindowsException;
    
    static int GetDriveType(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return GetDriveType0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native int GetDriveType0(long p0) throws WindowsException;
    
    static WindowsNativeDispatcher.DiskFreeSpace GetDiskFreeSpaceEx(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            WindowsNativeDispatcher.DiskFreeSpace diskFreeSpace = new WindowsNativeDispatcher.DiskFreeSpace();
            GetDiskFreeSpaceEx0(nativeBuffer.address(), diskFreeSpace);
            return diskFreeSpace;
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native void GetDiskFreeSpaceEx0(long p0, WindowsNativeDispatcher.DiskFreeSpace p1) throws WindowsException;
    
    static String GetVolumePathName(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return GetVolumePathName0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native String GetVolumePathName0(long p0) throws WindowsException;
    
    static native void InitializeSecurityDescriptor(long p0) throws WindowsException;
    
    static native void InitializeAcl(long p0, int p1) throws WindowsException;
    
    static int GetFileSecurity(String s, int n, long n2, int n3) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return GetFileSecurity0(nativeBuffer.address(), n, n2, n3);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native int GetFileSecurity0(long p0, int p1, long p2, int p3) throws WindowsException;
    
    static void SetFileSecurity(String s, int n, long n2) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            SetFileSecurity0(nativeBuffer.address(), n, n2);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    static native void SetFileSecurity0(long p0, int p1, long p2) throws WindowsException;
    
    static native long GetSecurityDescriptorOwner(long p0) throws WindowsException;
    
    static native void SetSecurityDescriptorOwner(long p0, long p1) throws WindowsException;
    
    static native long GetSecurityDescriptorDacl(long p0);
    
    static native void SetSecurityDescriptorDacl(long p0, long p1) throws WindowsException;
    
    static WindowsNativeDispatcher.AclInformation GetAclInformation(long n) {
        WindowsNativeDispatcher.AclInformation aclInformation = new WindowsNativeDispatcher.AclInformation();
        GetAclInformation0(n, aclInformation);
        return aclInformation;
    }
    
    private static native void GetAclInformation0(long p0, WindowsNativeDispatcher.AclInformation p1);
    
    static native long GetAce(long p0, int p1);
    
    static native void AddAccessAllowedAceEx(long p0, int p1, int p2, long p3) throws WindowsException;
    
    static native void AddAccessDeniedAceEx(long p0, int p1, int p2, long p3) throws WindowsException;
    
    static WindowsNativeDispatcher.Account LookupAccountSid(long n) throws WindowsException {
        WindowsNativeDispatcher.Account account = new WindowsNativeDispatcher.Account();
        LookupAccountSid0(n, account);
        return account;
    }
    
    private static native void LookupAccountSid0(long p0, WindowsNativeDispatcher.Account p1) throws WindowsException;
    
    static int LookupAccountName(String s, long n, int n2) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return LookupAccountName0(nativeBuffer.address(), n, n2);
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native int LookupAccountName0(long p0, long p1, int p2) throws WindowsException;
    
    static native int GetLengthSid(long p0);
    
    static native String ConvertSidToStringSid(long p0) throws WindowsException;
    
    static long ConvertStringSidToSid(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return ConvertStringSidToSid0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native long ConvertStringSidToSid0(long p0) throws WindowsException;
    
    static native long GetCurrentProcess();
    
    static native long GetCurrentThread();
    
    static native long OpenProcessToken(long p0, int p1) throws WindowsException;
    
    static native long OpenThreadToken(long p0, int p1, boolean p2) throws WindowsException;
    
    static native long DuplicateTokenEx(long p0, int p1) throws WindowsException;
    
    static native void SetThreadToken(long p0, long p1) throws WindowsException;
    
    static native int GetTokenInformation(long p0, int p1, long p2, int p3) throws WindowsException;
    
    static native void AdjustTokenPrivileges(long p0, long p1, int p2) throws WindowsException;
    
    static native boolean AccessCheck(long p0, long p1, int p2, int p3, int p4, int p5, int p6) throws WindowsException;
    
    static long LookupPrivilegeValue(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return LookupPrivilegeValue0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native long LookupPrivilegeValue0(long p0) throws WindowsException;
    
    static void CreateSymbolicLink(String s, String s2, int n) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        NativeBuffer nativeBuffer2 = asNativeBuffer(s2);
        try {
            CreateSymbolicLink0(nativeBuffer.address(), nativeBuffer2.address(), n);
        }
        finally {
            nativeBuffer2.release();
            nativeBuffer.release();
        }
    }
    
    private static native void CreateSymbolicLink0(long p0, long p1, int p2) throws WindowsException;
    
    static void CreateHardLink(String s, String s2) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        NativeBuffer nativeBuffer2 = asNativeBuffer(s2);
        try {
            CreateHardLink0(nativeBuffer.address(), nativeBuffer2.address());
        }
        finally {
            nativeBuffer2.release();
            nativeBuffer.release();
        }
    }
    
    private static native void CreateHardLink0(long p0, long p1) throws WindowsException;
    
    static String GetFullPathName(String s) throws WindowsException {
        NativeBuffer nativeBuffer = asNativeBuffer(s);
        try {
            return GetFullPathName0(nativeBuffer.address());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    private static native String GetFullPathName0(long p0) throws WindowsException;
    
    static native String GetFinalPathNameByHandle(long p0) throws WindowsException;
    
    static native String FormatMessage(int p0);
    
    static native void LocalFree(long p0);
    
    static native long CreateIoCompletionPort(long p0, long p1, long p2) throws WindowsException;
    
    static WindowsNativeDispatcher.CompletionStatus GetQueuedCompletionStatus(long n) throws WindowsException {
        WindowsNativeDispatcher.CompletionStatus completionStatus = new WindowsNativeDispatcher.CompletionStatus();
        GetQueuedCompletionStatus0(n, completionStatus);
        return completionStatus;
    }
    
    private static native void GetQueuedCompletionStatus0(long p0, WindowsNativeDispatcher.CompletionStatus p1) throws WindowsException;
    
    static native void PostQueuedCompletionStatus(long p0, long p1) throws WindowsException;
    
    static native void ReadDirectoryChangesW(long p0, long p1, int p2, boolean p3, int p4, long p5, long p6) throws WindowsException;
    
    static native void CancelIo(long p0) throws WindowsException;
    
    static native int GetOverlappedResult(long p0, long p1) throws WindowsException;
    
    static WindowsNativeDispatcher.BackupResult BackupRead(long n, long n2, int n3, boolean b, long n4) throws WindowsException {
        WindowsNativeDispatcher.BackupResult backupResult = new WindowsNativeDispatcher.BackupResult();
        BackupRead0(n, n2, n3, b, n4, backupResult);
        return backupResult;
    }
    
    private static native void BackupRead0(long p0, long p1, int p2, boolean p3, long p4, WindowsNativeDispatcher.BackupResult p5) throws WindowsException;
    
    static native void BackupSeek(long p0, long p1, long p2) throws WindowsException;
    
    static NativeBuffer asNativeBuffer(String owner) throws WindowsException {
        if (owner.length() > 1073741822) {
            throw new WindowsException("String too long to convert to native buffer");
        }
        int n = owner.length() << 1;
        int n2 = n + 2;
        NativeBuffer nativeBuffer = NativeBuffers.getNativeBufferFromCache(n2);
        if (nativeBuffer == null) {
            nativeBuffer = NativeBuffers.allocNativeBuffer(n2);
        }
        else if (nativeBuffer.owner() == owner) {
            return nativeBuffer;
        }
        WindowsNativeDispatcher.unsafe.copyMemory(owner.toCharArray(), (long)Unsafe.ARRAY_CHAR_BASE_OFFSET, null, nativeBuffer.address(), (long)n);
        WindowsNativeDispatcher.unsafe.putChar(nativeBuffer.address() + (long)n, '\0');
        nativeBuffer.setOwner(owner);
        return nativeBuffer;
    }
    
    private static native void initIDs();
    
    static {
        unsafe = Unsafe.getUnsafe();
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                System.loadLibrary("net");
                System.loadLibrary("nio");
                return null;
            }
        });
        initIDs();
    }
    
    static class FirstFile
    {
        private long handle;
        private String name;
        private int attributes;
        
        private FirstFile() {
        }
        
        public long handle() {
            return this.handle;
        }
        
        public String name() {
            return this.name;
        }
        
        public int attributes() {
            return this.attributes;
        }
    }
    
    static class FirstStream
    {
        private long handle;
        private String name;
        
        private FirstStream() {
        }
        
        public long handle() {
            return this.handle;
        }
        
        public String name() {
            return this.name;
        }
    }
    
    static class VolumeInformation
    {
        private String fileSystemName;
        private String volumeName;
        private int volumeSerialNumber;
        private int flags;
        
        private VolumeInformation() {
        }
        
        public String fileSystemName() {
            return this.fileSystemName;
        }
        
        public String volumeName() {
            return this.volumeName;
        }
        
        public int volumeSerialNumber() {
            return this.volumeSerialNumber;
        }
        
        public int flags() {
            return this.flags;
        }
    }
    
    static class DiskFreeSpace
    {
        private long freeBytesAvailable;
        private long totalNumberOfBytes;
        private long totalNumberOfFreeBytes;
        
        private DiskFreeSpace() {
        }
        
        public long freeBytesAvailable() {
            return this.freeBytesAvailable;
        }
        
        public long totalNumberOfBytes() {
            return this.totalNumberOfBytes;
        }
        
        public long totalNumberOfFreeBytes() {
            return this.totalNumberOfFreeBytes;
        }
    }
    
    static class AclInformation
    {
        private int aceCount;
        
        private AclInformation() {
        }
        
        public int aceCount() {
            return this.aceCount;
        }
    }
    
    static class Account
    {
        private String domain;
        private String name;
        private int use;
        
        private Account() {
        }
        
        public String domain() {
            return this.domain;
        }
        
        public String name() {
            return this.name;
        }
        
        public int use() {
            return this.use;
        }
    }
    
    static class CompletionStatus
    {
        private int error;
        private int bytesTransferred;
        private long completionKey;
        
        private CompletionStatus() {
        }
        
        int error() {
            return this.error;
        }
        
        int bytesTransferred() {
            return this.bytesTransferred;
        }
        
        long completionKey() {
            return this.completionKey;
        }
    }
    
    static class BackupResult
    {
        private int bytesTransferred;
        private long context;
        
        private BackupResult() {
        }
        
        int bytesTransferred() {
            return this.bytesTransferred;
        }
        
        long context() {
            return this.context;
        }
    }
}
